# Path & Name to Log- / Sensorfile(s)
Core="/sys/devices/platform/coretemp.0/"
i2c="/sys/bus/i2c/devices/0-002e/"
userDir="/var/services/homes/admin/AvrLogger/"

# IP for SNMP-Service:
SynoCom="syno"
Ip=127.0.0.1

# ----------------------------------------------------------
# --- !! Do not change the order !!!
# ----------------------------------------------------------
TabLoad=0;   TabLoadFlag=1   # AvrLoad
TabCpu=1;    TabCpuFlag=1    # AvrCpu
TabStatus=2; TabStatusFlag=1 # AvrStatus
TabEnv=3;    TabEnvFlag=1    # AvrEnvirenmentSensor
TabHmax=4;   TabHmaxFlag=1   # AvrHddMax
TabHmin=5;   TabHminFlag=1   # AvrHddMin
TabHavr=6;   TabHavrFlag=1   # AvrHddAvr  : Average
TabPwm=7;    TabPwmFlag=1    # AvrPwm
Tabi2c=8;    Tabi2cFlag=1    # Avri2c
TabLast=8

# ----------------------------------------------------------
# --- Init / create directory structure of User-AvrLogger-Home and
# --- copy default Files if not done yet.
# ----------------------------------------------------------
init_UserDir()
{
    # --- Status von "Benutzer-Home-Dienst" abfragen
    lib_checkUserHome

    # --- Ohne aktivierten "Benutzer-Home-Dienst" bleiben wir hier und meckern gelegentlich (alle 30s)
    until [ $userHomeFlag -eq 1 ]; do
        synodsmnotify @administrators "AvrLogger" "To complete the installation please enable the User-Home-Service"
        sleep 30
        lib_checkUserHome
    done

    # --- Prüfen ob das Home-Verzeichnis von "admin" auch wirklich vorhanden ist ...
    if [ -d "/var/services/homes/admin" ]
    then
        # --- Prüfen ob Home-Verzeichnis von AvrLogger bereits vorhanden ist ...
        if [ ! -d "/var/services/homes/admin/AvrLogger" ]
        then
            # --- Ordner und Unterordner erstellen.
            mkdir -p "${userDir}"{data/archive,tmp,config,log}
            cp "${workingDir}templates/log.conf.txt" "${userDir}config/log.conf"
            cp "${workingDir}templates/svg.conf.txt" "${userDir}config/svg.conf"
        else
            if [ ! -d "${userDir}data" ]
                then mkdir -p "${userDir}data{archive}"
            fi
            if [ ! -d "${userDir}tmp" ]
                then mkdir -p "${userDir}tmp"
            fi
            if [ ! -d "${userDir}config" ]; then
                mkdir -p "${userDir}config"
                cp "${workingDir}templates/log.conf.txt" "${userDir}config/log.conf"
                cp "${workingDir}templates/svg.conf.txt" "${userDir}config/svg.conf"
            fi
            if [ ! -d "${userDir}log" ]
                then mkdir -p "${userDir}log"
            fi
        fi
    else
        synodsmnotify @administrators "AvrLogger" "A fatal error has occurred - AvrLogger will now be terminated."
        exit 1
    fi

    LOGFILE="${userDir}log/messages"
    touch "$LOGFILE"
    ERROR="${userDir}log/error"
    touch "$ERROR"

    # --- recursive "alle" Ordner und Dateien: Benutzer und Dateirechte setzen
    chown -R admin:users "${userDir}"
    chmod -R 771 "${userDir}"

}

# ----------------------------------------------------------
# --- Initialize moving average filter
declare -A arr
arr_cnt=0
init_Array()
{
    local i j
	arr_row=$1
	arr_col=$2

	for ((i=0;i<=arr_row;i++)) do
		for ((j=0;j<=arr_col;j++)) do
	    	arr[$i,$j]=0
		done
	done
}

# ----------------------------------------------------------
# --- Performe Main-Initialization due Config-Update
# ----------------------------------------------------------
UpdateFlag=0
init_ConfigUpdate()
{
    lib_TimerStart
    UpdateFlag=1
    lib_Echo 0 "+---------------------------------------------------------------=[ Update initiated ]=-+"
    lib_Echo 0 "| AVR: Config changes detected - Settings are being updated ...                        |"
    lib_Echo 0 "+--------------------------------------------------------------------------------------+"

    if [ $LogConfFlag -eq 1 ]; then
        arr_cnt=0
        init_Init
        checkSensors
        checkEnv
        SvgConfFlag=1
    fi

    if [ $SvgConfFlag -eq 1 ]; then
        svg_Init "10"
        PlotRow=0; PlotCol=0
        minRowFlag=0
        maxRowFlag=0
        RowOffset=0
        svgTimeline=0
        NewLines=0
        init_PlotData $MaxLines  # Read log data from previous date
        svg_InitPlot
        svg_SVG 0

        s_time=$[$(date +%s)+$SvgTick]
        t_time=$[$(date +%s)+$LoopTick]
        LoopCount=0
        arrOffset=0
    fi

    lib_TimerStop
    lib_Echo 0 "| AVR: Config-Update has been processed" "$timerLap |"
    lib_Echo 0 "+--------------------------------------------------------------------------------------+"

    UpdateFlag=0
    LogConfFlag=0
    SvgConfFlag=0

    l_time=$[$(date +%s)+2]
}

# ----------------------------------------------------------
# --- Main-Initialization
# ----------------------------------------------------------
init_Init()
{
    if [ $ExecutionFlag -eq 0 ]; then
	    # --- Store PID
	    LogPID=$$
        echo "$LogPID" > "${workingDir}${Name}.pid"
    fi

    LogConf="${userDir}config/log.conf"
    LogDat="${userDir}tmp/${Name}.dat"
    LogTmp="${userDir}tmp/${Name}.tmp"
    SvgConf="${userDir}config/svg.conf"
    SvgFile="${userDir}data/${Name}.svg"

    # --- Path to Synology cpuinfo
    SynoCpuInfo="/proc/cpuinfo"
    # --- Path to Synology synoinfo.conf
	SynoInfo="/etc/synoinfo.conf"

    # ----------------------------------------------------------
	# --- Get some CPU Infos
    # ---> SynoCpu[0] = Manufacturer
    # ---> SynoCpu[1] = Product Collection
    # ---> SynoCpu[2] = Processor Number
    # ---> SynoCpu[3] = # of Cores
    # ---> SynoCpu[4] = # of Threads
    # ---> SynoCpu[5] = Base Frequency GHz
    read -r -a SynoCPU <<< $(cat /proc/syno_cpu_arch | sed 's/[ ]//g' | sed 's/[,]/ /g')
    SynoCPU[4]=$(cat "/proc/cpuinfo" | grep -c '^processor')
    SynoCPU[5]=$(cat "/proc/cpuinfo" | grep -m 1 "cpu MHz" | awk '{print $4/1000}')

    # ----------------------------------------------------------
	# --- Get some Syno Infos
	# ---> SynoModel[0] = Synology Product
	# ---> SynoModel[1] = Product Type ("RackStation" / "DiskStation")
	# ---> SynoModel[2] = Platform
	# ---> SynoModel[3] = Hostname
	# ---> SynoModel[4] = Uptime Sec.
	SynoModel[0]=$(GetValue $SynoInfo upnpmodelname)  # Synology Product "RS3617xs"
    SynoModel[1]=$(GetValue $SynoInfo product)        # RackStation / DiskStation
    SynoModel[2]=$(cat /proc/syno_platform)
    SynoModel[3]=$(cat /etc/hostname)
    SynoModel[4]=$(cat /proc/uptime | awk '{print $1}' | sed -r 's/([^\.]*)\..*/\1/')

    uDay=$[SynoModel[4]/86400]
    SynoModel[4]=$[SynoModel[4]-(uDay*86400)]
    uHour=$[SynoModel[4]/3600]
    SynoModel[4]=$[SynoModel[4]-(uHour*3600)]
    uMin=$[SynoModel[4]/60]
    SynoModel[4]=$[SynoModel[4]-(uMin*60)]
    uSec=${SynoModel[4]}
    SynoModel[4]="$(printf "%4d:%02d:%02d:%02d" "$uDay" "$uHour" "$uMin" "$uSec")"

    # --- FAN Speed
    Rpm=$(GetValue "$LogConf" "FanSpeed")


    # --- Get AvrTick / SvgTick Time
    LogBase=$(GetValue "$LogConf" "LogBase")
    case $LogBase in
    	 1) MaxLines=1200;  LoopTick=1; arrDiv=3;  arrMax=10; timeStep=60 ;;  #  1 x  3 x 1200  =   3600s =   1h
    	 2) MaxLines=1200;  LoopTick=2; arrDiv=3;  arrMax=10; timeStep=60 ;;  #  2 x  3 x 1200  =   7200s =   2h
    	 3) MaxLines=1200;  LoopTick=3; arrDiv=3;  arrMax=10; timeStep=60 ;;  #  3 x  3 x 1200  =  10800s =   3h
    	 6) MaxLines=1440;  LoopTick=3; arrDiv=5;  arrMax=10; timeStep=60 ;;  #  3 x  5 x 1440  =  21600s =   6h
    	12) MaxLines=1440;  LoopTick=3; arrDiv=10; arrMax=10; timeStep=60 ;;  #  3 x 10 x 1440  =  43200s =  12h
    	24) MaxLines=1440;  LoopTick=6; arrDiv=10; arrMax=10; timeStep=60 ;;  #  6 x 10 x 1440  =  86400s =  24h = 1d
    	48) MaxLines=2880;  LoopTick=6; arrDiv=10; arrMax=10; timeStep=90 ;;  #  6 x 10 x 2880  = 172800s =  48h = 2d
       168) MaxLines=10080; LoopTick=6; arrDiv=10; arrMax=10; timeStep=120 ;; #  6 x 10 x 10080 = 604800s = 168h = 7d
    	 *) LogBase=24
    	    MaxLines=1440; LoopTick=6; arrDiv=10; arrMax=10; timeStep=60 ;;
    esac

    # ----------------------------------------------------------
    # --- Flag: Enable SVG-Archive
	SvgArchive=$(GetValue "$LogConf" "SvgArchive")
    case $SvgArchive in
        "Base") SvgArchive=2; Rdate=$[$(date +%s)+(LogBase*3600)];;         # Time + LogBase*3600
         "Day") SvgArchive=1; Rdate=$(date -d "tomorrow 00:00:00" '+%s');;  # Tomorrow at Midnight
             *) SvgArchive=0; Rdate=$(date -d "tomorrow 00:00:00" '+%s');;  # Tomorrow at Midnight
    esac

    # --- Moving Avarage Array initialisieren
    init_Array $arrMax $TabLast
    # --- Config-Timestamp initialisieren
    lib_ConfigTimestamp
    ConfStatFlag=0
    LogConfFlag=0
    SvgConfFlag=0

    SvgTick=$(GetValue "$LogConf" "SvgTick")

	# --- Environmental sensor Name, Ip & OID
	LogEnvIp=$(GetValue "$LogConf" "LogEnvIp")
    if [ ${#LogEnvIp} -eq 0 ]
    then
        LogEnvFlag=0
        TabEnvFlag=0
        LogEnvStatus="Off"
        LogEnvName=""
	    LogEnvCom=""
    else
        LogEnvFlag=1
        LogEnvStatus="On"
	    LogEnvCom=$(GetValue "$LogConf" "LogEnvCom")
        LogEnvName=$(GetValue "$LogConf" "LogEnvName")
	    LogEnvOID=$(GetValue "$LogConf" "LogEnvOID")
    fi

    if [ $UpdateFlag -eq 0 ]; then
        lib_Echo 0 "+--------------------------------------------------------------=[ AvrLogger StartUp ]=-+"
    fi

    # --- Prepare Start-Stop-Script
    # --- ToDo: Remove this when AvrLogger is SPK
    LogService=$(GetValue "$LogConf" "LogService")
    src="${workingDir}templates/start-stop-status.sh"
    dst="/usr/local/etc/rc.d/${Name}_start.sh"
    
    if [ "$LogService" == "Yes" ]; then
        cp "$src" "$dst"
        lib_PutValue "$dst" "workingDir"     "$workingDir"
        lib_PutValue "$dst" "workingName"    "$Name"
        lib_PutValue "$dst" "workingERROR"   "$ERROR"
        lib_PutValue "$dst" "workingLOGFILE" "$LOGFILE"
        chmod 755 "$dst"
        lib_Echo 0 "| Avr: Startup-Script copied to /usr/local/etc/rc.d/" "|"
    fi

    if [ "$LogService" == "No" ]; then
        if [ -e "$dst" ]; then
            rm "$dst"
            lib_Echo 0 "| Avr: Startup-Script removed from /usr/local/etc/rc.d/" "|"
        fi
    fi
}

# ----------------------------------------------------------
# Divide given avr.dat back to Array an prepare for SVG
# --- $1 = Max. Lines in .dat
# ----------------------------------------------------------
PlotRow=0; PlotCol=0
declare -A plotData
declare -A plotTime
init_PlotData()
{
    lib_TimerStart
    local Range=$MaxLines; VerMax=$MaxLines; VerMin=0
    local PriMin=$(GetValue "$SvgConf" "PriMin") PriMax=$(GetValue "$SvgConf" "PriMax")
    local SecMin=$(GetValue "$SvgConf" "SecMin") SecMax=$(GetValue "$SvgConf" "SecMax")
    local temp
    PlotVerScaleX=$[(pw*1000)/Range]

    # --- Plot Parameters --- Secondary Axis %
    y_Min[TabLoad]=$SecMin;   y_Max[TabLoad]=$SecMax
    y_Min[TabPwm]=$SecMin;    y_Max[TabPwm]=$SecMax
    # --- Plot Parameters --- Primary Axis °C
    y_Min[TabCpu]=$PriMin;    y_Max[TabCpu]=$PriMax
    y_Min[TabStatus]=$PriMin; y_Max[TabStatus]=$PriMax
    y_Min[TabEnv]=$PriMin;    y_Max[TabEnv]=$PriMax
    y_Min[TabHmax]=$PriMin;   y_Max[TabHmax]=$PriMax
    y_Min[TabHmin]=$PriMin;   y_Max[TabHmin]=$PriMax
    y_Min[TabHavr]=$PriMin;   y_Max[TabHavr]=$PriMax
    y_Min[Tabi2c]=$PriMin;    y_Max[Tabi2c]=$PriMax

    for ((i=0;i<=TabLast;i++)) do
        y_Range[$i]=$[y_Max[$i]-y_Min[$i]]
        y_Min[$i]=$[y_Min[$i]*10]
        y_Scale[$i]=$[(ph*10)/y_Range[$i]]
    done

    lib_trimFile $1 "$LogDat" "$LogTmp"

	if [ -e "$LogTmp" ]
	then
        minRowFlag=1
        n=$(wc -l "$LogTmp" | cut -d" " -f1)

        for ((i=0;i<n;i++)) do
        	plotTime[$i]=$[(PlotVerScaleX*i)/1000]
            read -r tmp
            tmp=($tmp)
            for ((j=0;j<=TabLast;j++)) do
                plotData[$i,$j]=$[((${tmp[$j]}-${y_Min[$j]})*${y_Scale[$j]})/100 ]
            done
        done < "$LogTmp"

	    PlotRow=$[i-1]
	fi

    if [ $PlotRow -eq $[${1}-1] ]; then maxRowFlag=1; fi

    lib_TimerStop
    if [ $PlotRow -eq 0 ]
       then lib_Echo 0 "| AVR: No Plot-Data imported" "|"
       else lib_Echo 0 "| AVR: Plot-Data imported : $[PlotRow+1]" "$timerLap |"
   fi
}

# ----------------------------------------------------------
# Print a list of importand Disk-Informations.
# ----------------------------------------------------------
declare -A SynoSmart
SynoDisks=0
init_ListDisk()
{
    local ID
    local smart

    # ---------------------------------------------------
    # --- Collect Disk's Data
    # ---------------------------------------------------
    lib_TimerStart

    lib_Echo 0 "| ID      | Type | Model                     | Serial          | Hours | °C   | Loop   |"
    lib_Echo 0 "+---------+------+---------------------------+-----------------+-------+------+--------+"

	for ID in `/bin/ls /dev/sd?`; do
        lib_TimerStart

        smart=`/usr/bin/smartctl -d ata -i $ID`
        # --- "dev/sd?"
        SynoSmart[$SynoDisks,0]="$ID"
        # --- "disk id"
        SynoSmart[$SynoDisks,1]=$[$(printf '%d\n' "'${ID:7:1}")-96]
        # --- "disk type"
        SynoSmart[$SynoDisks,2]=$(snmpwalk -v 2c  -c $SynoCom -O qv $Ip 1.3.6.1.4.1.6574.2.1.1.4.$SynoDisks | sed 's/^[^"]*"//' | sed  's/".*//')
        # --- "disk model"
		SynoSmart[$SynoDisks,3]=$(echo "$smart" | sed -n -e 's|^.*Device Model:||p' | sed -e 's/^[ \t]*//')
        # --- "disk serial"
		SynoSmart[$SynoDisks,4]=$(echo "$smart" | sed -n -e 's|^.*Serial Number:||p' | sed -e 's/^[ \t]*//')

		smart=`/usr/bin/smartctl -d ata -A $ID`
        # --- "SMART   9: Power_On_Hours"
		SynoSmart[$SynoDisks,5]=$(echo "$smart" | grep "Power_On_Hours" | awk '{print $10}')
        # --- "SMART 190: Airflow_Temperature_Cel
		SynoSmart[$SynoDisks,6]=$(echo "$smart" | grep "Airflow_Temperature_Cel" | awk '{print $10}')
        # --- "SMART 194: Temperature_Celsius
		SynoSmart[$SynoDisks,7]=$(echo "$smart" | grep "Temperature_Celsius" | awk '{print $10}')

        if [ "${SynoSmart[$SynoDisks,7]}" == "" ]; then
            SynoSmart[$SynoDisks,7]=${SynoSmart[$SynoDisks,6]}
        fi

        lib_TimerStop
        lib_Echo 0 "$(printf "| Disk%3d | %-4.4s | %-25.25s | %-15.15s | %-5.5s | %2d°C |" ${SynoSmart[$SynoDisks,1]} "${SynoSmart[$SynoDisks,2]}" "${SynoSmart[$SynoDisks,3]}" "${SynoSmart[$SynoDisks,4]}" "${SynoSmart[$SynoDisks,5]}" ${SynoSmart[$SynoDisks,6]})" "$timerLap |"
        SynoDisks=$[SynoDisks+1]
	done

    lib_Echo 0 "+---------+------+---------------------------+-----------------+-------+------+--------+"
    lib_TimerStop
    lib_Echo 0 "| SYS: -"$SynoDisks"- Disk Infos collected" "$timerLap |"
}

init_Log()
{
    local avrDate=$(date '+%Y.%m.%d')
    local avrTime=$(date '+%H:%M:%S')
    local AvrTick=$[LoopTick*arrDiv]

    lib_Echo 0           "+=[ AvrLogger v2 ]=--------------------------------------------------------------------+"
    lib_Echo 0 "$(printf "| AvrVer. =  %8.8s  SvgVer. =  %8.8s  Date    =  %10.10s                      |" "$Version" "$SvgVersion" "$avrDate")"
    lib_Echo 0 "$(printf "| AvrTick =    %3.3sSec  SvgTick =    %3.3sSec  Time    =    %8.8s                      |" "$AvrTick" "$SvgTick" "$avrTime")"
#   lib_Echo 0 "$(printf "| AvrLog  =       %3.3s  AvrMode =    %6.6s  SvgLog  =         %3.3s  |" "$LogStatus" "$AvrLogMode" "$SvgStatus")"
    lib_Echo 0           "+=[ Sensors ]=----------------=[ Environment ]=----------------------------------------+"
    lib_Echo 0 "$(printf "|                FAN Sens. = %2d  ENV  = %3.3s   IP = %15.15s                     |" "$fan_count" "$LogEnvStatus" "$LogEnvIp")"
    lib_Echo 0 "$(printf "| CPU Sen. = %2d  i2c Sens. = %2d  Name = %26.26s                     |" "$cpu_count" "$i2c_count" "$LogEnvName")"
    lib_Echo 0           "+=[ Syno ]=----------------=[ CPU ]=---------------------------------------------------+"
    lib_Echo 0 "$(printf "| Model    = %15.15s  Manufacturer = %17.17s                         |" "${SynoModel[0]}" "${SynoCPU[0]}")"
    lib_Echo 0 "$(printf "| Product  = %15.15s  Collection   = %17.17s                         |" "${SynoModel[1]}" "${SynoCPU[1]}")"
    lib_Echo 0 "$(printf "| Platform =   %13.13s  Name         = %17.17s                         |" "${SynoModel[2]}" "${SynoCPU[2]}")"
    lib_Echo 0 "$(printf "| Hostname = %15.15s  Clock        =           %1.2fGHz                         |" "${SynoModel[3]}" "${SynoCPU[5]}")"
    lib_Echo 0 "$(printf "| Uptime   =   %13.13s  Cores    =  %2d   Threads   =  %2d                         |" "${SynoModel[4]}" "${SynoCPU[3]}" "${SynoCPU[4]}")"
    lib_Echo 0           "+=[ Disk's ]=--------------------------------------------------------------------------+"
}

