# ----------------------------------------------------------
# --- Trap STRG-C Handle
# ----------------------------------------------------------
lib_STRG_C()
{
    lib_Echo 0 " "
    lib_Echo 0 "+--------------------------------------------------------------------------------------+"
    lib_Echo 0 "| AVR: The AvrLogger2 execution has been terminated.                                   |"
    lib_Echo 0 "+----------------------------------=[ TERMINATED ]=------------------------------------+"

    # ----------------------------------------------------------
    # --- Remove PID
    if [[ -e "${workingDir}${Name}.pid" && $ExecutionFlag -eq 0 ]]; then rm "${workingDir}${Name}.pid"; fi

    exit 0
}

# ----------------------------------------------------------
# --- Prüfen ob der "Benutzer-Home-Dienst" aktiviert ist.
# -> userHomeFlag = 0 wenn "nicht" aktiviert
# -> userHomeFlag = 1 wenn aktiviert
lib_checkUserHome()
{
    userHomeFlag=$(GetValue "/etc/synoinfo.conf" "userHomeEnable")
    if [[ "$userHomeFlag" == "yes" ]]
        then userHomeFlag=1
        else userHomeFlag=0
    fi
}

# ----------------------------------------------------------
# --- Check availability of a OID and returns result
# ----------------------------------------------------------
# --- $1 = Community-String
# --- $2 = Agend IP-Adress
# --- $3 = OID
# --- $4 = OID-Name (for ERROR-Log)
# --- Retur Codes:
# >= 0 = Query returned a valid result
#   -1 = snmpwalk: Unknown host    = Commnity-String or IP are not correct
#   -2 = Timeout: No Response      = Commnity-String or IP are not correct
#   -3 = No Such Object            = OID not found
lib_getOID()
{
    local TestOID=""
    local ErrOID=0

    # --- Get OID-Value or return error message
    TestOID=$(snmpwalk -v 2c  -c $1 -O qv $2 $3 2>&1)

    if [ -z "${TestOID/*"Unknown host"*}" ]; then
        ErrOID=-1
        TestOID="Unknown host"
    fi

    if [ -z "${TestOID/*"Timeout"*}" ]; then
        ErrOID=-2
        TestOID="Timeout"
    fi

    if [ -z "${TestOID/*"No Such Object"*}" ]; then
        ErrOID=-3
        TestOID="No Such Object"
    fi

    if [ $ErrOID -lt 0 ]; then
        lib_Echo 1 "OID: '$4' did not respond correctly"
        lib_Echo 1 "OID: Error:    : $TestOID"
        lib_Echo 1 "OID: OID       : $3"
        lib_Echo 1 "OID: IP        : $2"
        lib_Echo 1 "OID: Community : $1"
        TestOID=0
    fi

    echo "$TestOID"
}

# ----------------------------------------------------------
# --- echo Messages to $ERROR and Terminal
# --- $1 = Message type:
# ---      0 = Standart Message to Terminal and $LOGFILE
# ---      1 = ERROR    Message to Terminal and $ERROR
# ---      2 = DEBUG    Debug-Message only to Terminal
# --- $2 = Message- / Outputstring
# --- $3 = Second Message ($timerLap)
lib_Echo()
{
    local tLen tLog lenLog
    local type

    case $1 in
    	 0) type=0; lenLog=88 ;;    # $LOGFILE
    	 1) type=1; lenLog=160 ;;   # $ERROR
    	 2) type=2; lenLog=88 ;;    # Only to Terminal
    	 *) # Unknown type of message -> redirect to $LOGFILE AND $ERROR
    	    type=3; lenLog=160 ;;
    esac

    if [ "$#" -gt 2 ]
    then
        # Bring string to length
        tLen=$[lenLog-${#3}-1]
        tLog=${2:0:${tLen}}
    #   tLen=$[lenLog-${#tLog}-${#3}-1]
        tLen=$[lenLog-${#tLog}-${#3}]
        tLog="$(printf "%s%${tLen}s%s" "$tLog" " " "$3")"
    else
        # Bring string to length
        tLog=${2:0:${lenLog}}
    fi

    if [[ $type -eq 0 || $type -eq 3 ]]; then echo "$(date '+%Y.%m.%d %H:%M:%S') $tLog" >> "$LOGFILE"; fi
    if [[ $type -eq 1 || $type -eq 3 ]]; then echo "$(date '+%Y.%m.%d %H:%M:%S') $tLog" >> "$ERROR"; fi
    if [ $DebugFlag -eq 1 ]; then echo "$tLog" > /dev/tty; fi

}

# ----------------------------------------------------------
# Add a new Row to plotData-Array.
# If necessary, increment RowOffset (Ring-Buffer)
# --- $1 = New Content
# --- $2 = Max. Rows of Array
# ----------------------------------------------------------
minRowFlag=0
maxRowFlag=0
RowOffset=0
NewLines=0
lib_AddPlot()
{
    local i j

    if [ $maxRowFlag -eq 1 ]; then
        # ----------------------------------------------------------
        # --- Inc. Offset of Ring-Buffer
        if [ $RowOffset -lt $[${1}-1] ]
            then RowOffset=$[RowOffset+1]
            else RowOffset=0
        fi
    fi
    # ----------------------------------------------------------
    if [ $minRowFlag -eq 0 ]
    then
        minRowFlag=1
    else
        if [ $maxRowFlag -eq 0 ]; then
            if [ $PlotRow -lt $[${1}-1] ]
                then PlotRow=$[PlotRow+1]
                else maxRowFlag=1; RowOffset=$[RowOffset+1]
            fi
        fi
    fi
    # ----------------------------------------------------------
    if [ $maxRowFlag -eq 0 ]; then  plotTime[$PlotRow]=$[(PlotVerScaleX*PlotRow)/10]; fi

    j=$[(PlotRow+RowOffset) % $1]
    for ((i=0;i<=TabLast;i++)) do
        plotData[$j,$i]=$[((${arr[0,$i]}-${y_Min[$i]})*${y_Scale[$i]})/100]
    done
    # ----------------------------------------------------------

    NewLines=$[NewLines+1]
}

# ----------------------------------------------------------
# --- Add a New Line to File. Limit number of lines.
# --- $1 = Destination-File
# --- $2 = Number of the line to be deleted
# --- $3 = Max Linenumbers in Destination-File
lib_AddLine()
{
    local n i
    local pine=${arr[0,0]}

    for ((i=1;i<=TabLast;i++)) do
        pine+=" "${arr[0,$i]}
    done

    if [ ! -e "$1" ]
    then
        echo "$pine" > "$1"
    	chmod 666 "$1"
    	return
    else
        echo "$pine" >> "$1"
    fi

    if [ $3 -gt 0 ]; then
        n=$(wc -l "$1" | cut -d" " -f1)

        if [ $n -le $((${3}+${2}-1)) ]; then return; fi

        n=$[(n-(${3}+${2}-1))+${2}-1]
        sed -i "${2},${n}d" "$1"
    fi
}

# ----------------------------------------------------------
# --- Check if an instance of AvrLogger is already running
# ----------------------------------------------------------
# --- $1 = Script name without extension
ExecutionFlag=0
lib_CheckPid()
{
    local chkPID
    local listPID
    local PID
    # --- Check AvrLogger PID
    if [ -e "${workingDir}${1}.pid" ]; then
        chkPID=`cat "${workingDir}${1}.pid"`
       	listPID=`ps -e | grep "${1}.sh" | grep -v grep | awk '{print $1;}'`
       	for PID in $listPID; do
           	if [ $PID -eq $chkPID ]
           	then
               	ExecutionFlag=1
    			break
           	fi
       	done
    fi
}

# ----------------------------------------------------------
# --- Get Time-Lap in nanoseconds
timerCount=0
lib_TimerStart()
{
    timerStart[$timerCount]=$(date +%s%N)
    timerCount=$[timerCount+1]
}

# ----------------------------------------------------------
# --- Write passed time since TimerStart in timerLap (milliseconds)
timerLap=""
lib_TimerStop()
{
    timerCount=$[timerCount-1]

    timerLap=$(($(date +%s%N)-${timerStart[$timerCount]}))
    timerLap=$(awk '{print $0/1000000000}' <<< $timerLap)
    timerLap="$(printf "%6.3f" $timerLap)"
}

# ----------------------------------------------------------
# --- Check if config files have been changed
# ----------------------------------------------------------
# --- Result is a BitMask:
# --- 00 = All unchanged
# --- 01 = LogConf changed
# --- 10 = SvgConf changed
# --- 11 = LogConf & SvgConf changed
LogConfStamp=""
LogConfFlag=0
SvgConfStamp=""
SvgConfFlag=0
ConfStatFlag=0
lib_ConfigTimestamp()
{
    local LogTime=$(stat -c %y "$LogConf")
    local SvgTime=$(stat -c %y "$SvgConf")

    if [ "$LogConfStamp" != "$LogTime" ]; then
        ConfStatFlag=1
        LogConfFlag=1
        LogConfStamp=$LogTime
    fi

    if [ "$SvgConfStamp" != "$SvgTime" ]; then
        ConfStatFlag=1
        SvgConfFlag=1
        SvgConfStamp=$SvgTime
    fi
}

# ----------------------------------------------------------
# --- Cut a file from the end to numbered lines
# ----------------------------------------------------------
# --- $1 = Number of Lines
# --- $2 = Source-File
# --- $3 = Destination-File
lib_trimFile()
{
    local n

	if [ ! -e "$2" ]; then return; fi

    # ----------------------------------------------------------
    # --- Get number of lines; if $n <= Numer of Lines; Return
    cp "$2" "$3"
    n=$(wc -l "$3" | cut -d" " -f1)
    if [ $n -le $1 ]; then return; fi

    n=$[n-$1]
    sed -i "1,${n}d" "$3"
}

# ----------------------------------------------------------
# --- Check Timer and archive SVG graphic
# ----------------------------------------------------------
# --- $1 = File to archive
# --- $2 = Timer-Mode
Rdate=$(date +%s)
lib_SvgArchive()
{
    # --- Only if there is something to archive
	if [[ ! -e "$1" || $(date +%s) -lt $Rdate ]]; then return; fi
    lib_TimerStart

    case $2 in
        1) Rdate=$(date -d "tomorrow 00:00:00" '+%s');;     # Tomorrow at Midnight
        2) Rdate=$[$(date +%s)+(LogBase*3600)];;            # Time + LogBase*3600
        *) Rdate=$(date -d "tomorrow 00:00:00" '+%s');;     # Tomorrow at Midnight
    esac


    local aTime="$(date '+%Y.%m.%d - %H.%M')"
    cp "$1" "${userDir}data/archive/${aTime} log.svg"
    aTime="$(date -d @"$Rdate" '+%Y.%m.%d %H:%M:%S')"
    lib_TimerStop
    lib_Echo 0 "| AVR: SVG graphic has been archived. Next : $aTime" "$timerLap |"
}

# ----------------------------------------------------------
# --- Copy "single" quoted Key to "all" nonquoted Key's
# --- $1 = Source-File
# --- $2 = Destination-File
# --- $3 = Key
# ----------------------------------------------------------
lib_CpySvg()
{
    sed -i 's|'"$3"'|'"$(GetValue "$1" "$3")"'|g' "$2"
}

# ----------------------------------------------------------
# --- Change "any" Key's (without quoting)
# --- $1 = File
# --- $2 = Key
# --- $3 = Value
# ----------------------------------------------------------
lib_PutSvg()
{
    sed -i 's|'"$2"'|'"$3"'|g' "$1"
}

# ----------------------------------------------------------
# --- Change a "single" Key quoted between "
# --- $1 = File
# --- $2 = Key
# --- $3 = Value
# ----------------------------------------------------------
lib_PutValue()
{
    sed -i '/^'"$2"'.*$/ s|\(\"\).*\(\"\)|\1'"$3"'\2|' "$1"
}

# ----------------------------------------------------------
# --- Schlüssel aus Config-File lesen
# --- $1 = Config-File
# --- $2 = Key
# ----------------------------------------------------------
GetValue()
{
    echo "$(grep -m 1 "^$2" "$1" | sed 's/^[^"]*"\([^"]*\)".*/\1/g')"
}
