# ----------------------------------------------------------
# --- Check availability of Environmental sensor
# ----------------------------------------------------------
checkEnv()
{
    local LogEnvErr

    if [ $LogEnvFlag -eq 1 ]; then
        LogEnvErr=$(checkOID "$LogEnvCom" "$LogEnvIp" "$LogEnvOID")
        if [ $LogEnvErr -lt 0 ]; then
            lib_Echo 1 "AVR: Env.-Sensor did not responce: Retry in 60 seconds"
            sleep 60
            LogEnvErr=$(checkOID "$LogEnvCom" "$LogEnvIp" "$LogEnvOID")
            if [ $LogEnvErr -lt 0 ]; then
                TabEnvFlag=0
                LogEnvFlag=0
                lib_Echo 1 "AVR: Env.-Sensor did not responce: Temporarily disabled!"
            fi
        fi
    fi
}
# ----------------------------------------------------------
# --- Check availability of a OID and returns result
# ----------------------------------------------------------
# --- $1 = Community-String
# --- $2 = Agend IP-Adress
# --- $3 = OID
# --- Retur Codes:
# >= 0 = Query returned a valid result
#   -1 = snmpwalk: Unknown host    = Commnity-String or IP are not correct
#   -2 = Timeout: No Response      = Commnity-String or IP are not correct
#   -3 = No Such Object            = OID not found
checkOID()
{
    local TestOID=""

    # --- Get OID-Value or return error message
    TestOID=$(snmpwalk -v 2c  -c $1 -O qv $2 $3 2>&1)

    if [ -z "${TestOID/*"Unknown host"*}" ]; then TestOID=-1; fi
    if [ -z "${TestOID/*"Timeout"*}" ]; then TestOID=-2; fi
    if [ -z "${TestOID/*"No Such Object"*}" ]; then TestOID=-3; fi

    echo "$TestOID"
} # --- End of [checkOID]

# ----------------------------------------------------------
# --- Check availability of Synology Sensors
# ----------------------------------------------------------
# CPU-Sensoren:
declare -a cpu_name
declare -a cpu_value
cpu_count=0

# Lüfter-Sensoren
declare -a fan_name
declare -a fan_value
fan_count=0

# i2c Thermal-Sensoren
declare -a i2c_name
declare -a i2c_value
i2c_count=0

checkSensors()
{
    # ----------------------------------------------------------
    # --- Determine the number of available CPU-Thermal sensors
    # ----------------------------------------------------------
    for (( c=1; c<=10; c++)) do
        if [ -e "${Core}temp${c}_input" ]; then
            cpu_count=$[cpu_count+1]
            cpu_name[$cpu_count]=$[c]
        fi
    done

    # ----------------------------------------------------------
    # --- Determine the number of available FAN-RPM sensors
    # ----------------------------------------------------------
    for (( c=1; c<=10; c++)) do
        if [ -e "${i2c}fan${c}_input" ]; then
            fan_count=$[fan_count+1]
            fan_name[$fan_count]=$[c]
        fi
    done
    if [ $fan_count -eq 0 ]; then TabPwmFlag=0; fi

    # ----------------------------------------------------------
    # --- Determine the number of available i2c-Thermal sensors
    # ----------------------------------------------------------
    for (( c=1; c<=10; c++)) do
        if [ -e "${i2c}temp${c}_input" ]; then
            i2c_count=$[i2c_count+1]
            i2c_name[$i2c_count]=$[c]
        fi
    done
    if [ $i2c_count -eq 0 ]; then Tabi2cFlag=0; fi

} # --- End of [checkSensors]
